@@ -44,37 +44,6 @@ exception Not_audio
 exception False
 exception Utf8_failure of string
 
-(* Dummy registration function for 
- * user compiling with ocaml < 3.11.2 *)
-let register_printer _ = ()
-
-module type StringWrapper =
-sig
-  include module type of String
-  val uppercase : string -> string
-end
-
-module StringWrapper : StringWrapper =
-struct
-  let uppercase _ = raise Not_implemented
-  let () =
-    try ignore(uppercase "") with Not_implemented -> ()
-  include String
-end
-
-module StringCompat =
-struct
-  let uppercase_ascii = StringWrapper.uppercase
-  let () =
-    try ignore(uppercase_ascii "") with Not_implemented -> ()
-  include String
-end
-
-(* Now open Printexc, 
- * overriding register_printer
- * if present *)
-open Printexc
-
 (* New register exception printer *)
 let string_of_exc e =
   let f s = Some s in
@@ -109,7 +78,7 @@ let string_of_exc e =
     | Utf8_failure s      -> f (Printf.sprintf "UTF8 failure in string: %S" s)
     | _                   -> None
 
-let () = register_printer string_of_exc
+let () = Printexc.register_printer string_of_exc
 
 let _ =
   Callback.register_exception "vorbis_exn_invalid_parameters" Invalid_parameters;
@@ -185,7 +154,7 @@ let split_comment comment =
       String.index_from comment 0 '='
     in
     let c1 =
-      StringCompat.uppercase_ascii (String.sub comment 0 equal_pos)
+      String.uppercase_ascii (String.sub comment 0 equal_pos)
     in
     let c2 =
       String.sub comment (equal_pos + 1) ((String.length comment) - equal_pos - 1)
@@ -219,7 +188,7 @@ struct
          (fun n ->
             let buf = Bytes.create n in
             let r = Unix.read fd buf 0 n in
-            buf, r)
+            Bytes.unsafe_to_string buf, r)
          (fun n cmd -> Unix.lseek fd n cmd)
          (fun () -> Unix.lseek fd 0 Unix.SEEK_CUR)
       with
