@@ -87,12 +87,6 @@ static void raise_err(int ret)
   }
 }
 
-static void check_err(int ret)
-{
-  if (ret)
-    raise_err(ret);
-}
-
 /**** Decoding *****/
 
 typedef struct
@@ -197,7 +191,7 @@ CAMLprim value ocaml_vorbis_synthesis_init(value packet, value packet2, value pa
 
   decoder_t *vt = malloc(sizeof(decoder_t));
   if (vt == NULL)
-    caml_failwith("malloc");
+    caml_raise_out_of_memory();
 
   vorbis_info_init(&vt->vi);
   vorbis_comment_init(&vt->vc);
@@ -261,8 +255,8 @@ CAMLprim value ocaml_vorbis_decode_pcm(value vorbis_state, value stream_state, v
         break;
       if (samples > len - total_samples)
         samples = len - total_samples;
-      if (Wosize_val(buf) < vi->channels)
-        caml_raise_constant(*caml_named_value("vorbis_exn_invalid"));
+      if (Wosize_val(buf) != vi->channels)
+        caml_raise_constant(*caml_named_value("vorbis_exn_invalid_channels"));
       for (i=0 ; i<vi->channels ; i++)
       {
         chan = Field(buf,i);
@@ -281,10 +275,22 @@ CAMLprim value ocaml_vorbis_decode_pcm(value vorbis_state, value stream_state, v
       CAMLreturn(Val_int(total_samples));
 
     ret = ogg_stream_packetout(os,&op);
+    /* returned values are:
+     * 1: ok
+     * 0: not enough data. in this case
+     *    we return the number of samples
+     *    decoded if > 0 and raise 
+     *    Ogg_not_enough_data otherwise
+     * -1: out of sync */
     if (ret == 0)
-      CAMLreturn(Val_int(total_samples)); 
-    if (ret < 0)
-      raise_err(ret);
+    {
+      if (total_samples > 0)
+        CAMLreturn(Val_int(total_samples)); 
+      else
+        caml_raise_constant(*caml_named_value("ogg_exn_not_enough_data"));
+    }
+    if (ret == -1)
+      caml_raise_constant(*caml_named_value("ogg_exn_out_of_sync"));
 
     caml_enter_blocking_section();
     ret = vorbis_synthesis(vb,&op);
@@ -343,10 +349,14 @@ CAMLprim value ocaml_vorbis_analysis_init(value channels, value rate, value max_
 {
   encoder_t *enc = malloc(sizeof(encoder_t));
   value ans;
+  int err;
 
   vorbis_info_init(&enc->vi);
-  /* TODO: clear info on error */
-  check_err(vorbis_encode_init(&enc->vi, Int_val(channels), Int_val(rate), Int_val(max_bitrate), Int_val(nominal_bitrate), Int_val(min_bitrate)));
+  err = vorbis_encode_init(&enc->vi, Int_val(channels), Int_val(rate), Int_val(max_bitrate), Int_val(nominal_bitrate), Int_val(min_bitrate));
+  if (err) {
+    vorbis_info_clear(&enc->vi);
+    raise_err(err);  
+  }
   vorbis_analysis_init(&enc->vd, &enc->vi);
   vorbis_block_init(&enc->vd, &enc->vb);
   ans = caml_alloc_custom(&encoder_ops, sizeof(encoder_t*), 1, 0);
@@ -359,10 +369,14 @@ CAMLprim value ocaml_vorbis_analysis_init_vbr(value channels, value rate, value
 {
   encoder_t *enc = malloc(sizeof(encoder_t));
   value ans;
+  int err;
 
   vorbis_info_init(&enc->vi);
-  /* TODO: clear info on error */
-  check_err(vorbis_encode_init_vbr(&enc->vi, Int_val(channels), Int_val(rate), Double_val(quality)));
+  err = vorbis_encode_init_vbr(&enc->vi, Int_val(channels), Int_val(rate), Double_val(quality));
+  if (err) {
+    vorbis_info_clear(&enc->vi);
+    raise_err(err);
+  }
   vorbis_analysis_init(&enc->vd, &enc->vi);
   vorbis_block_init(&enc->vd, &enc->vb);
   ans = caml_alloc_custom(&encoder_ops, sizeof(encoder_t*), 1, 0);
@@ -390,7 +404,8 @@ CAMLprim value ocaml_vorbis_analysis_headerout(value vdsp, value comments)
   CAMLparam2(vdsp, comments);
   CAMLlocal4(ret,p1,p2,p3);
   vorbis_dsp_state *vd = Enc_dsp_state_val(vdsp);
-  vorbis_comment vc;
+  
+vorbis_comment vc;
   ogg_packet header, header_comm, header_code;
   int i;
 
@@ -408,6 +423,13 @@ CAMLprim value ocaml_vorbis_analysis_headerout(value vdsp, value comments)
   CAMLreturn(ret);
 }
 
+CAMLprim value ocaml_vorbis_encode_get_channels(value vdsp)
+{
+  CAMLparam1(vdsp);
+  encoder_t *enc = Encoder_val(vdsp);
+  CAMLreturn(Val_int(enc->vi.channels));
+}
+
 CAMLprim value ocaml_vorbis_encode_float(value vdsp, value vogg, value data, value _offs, value _len)
 {
   CAMLparam3(vdsp, vogg, data);
@@ -421,11 +443,13 @@ CAMLprim value ocaml_vorbis_encode_float(value vdsp, value vogg, value data, val
   float **vorbis_buffer;
   int c, i;
   value datac;
+  int channels = enc->vi.channels;
 
-  /* TODO: check for consistency of data */
+  if (Wosize_val(data) != channels) 
+    caml_raise_constant(*caml_named_value("vorbis_exn_invalid_channels"));
 
   vorbis_buffer = vorbis_analysis_buffer(vd, len);
-  for(c = 0; c < Wosize_val(data); c++)
+  for(c = 0; c < channels; c++)
   {
     datac = Field(data, c);
     for(i = 0; i < len; i++)
@@ -470,7 +494,6 @@ typedef struct
   int bitstream;
   value read_func;
   value seek_func;
-  value close_func;
   value tell_func;
 } myvorbis_dec_file_t;
 
@@ -480,22 +503,12 @@ static void finalize_dec_file(value _df)
 {
   myvorbis_dec_file_t *df = Decfile_val(_df);
 
-  if (df->ovf)
-  {
-    caml_enter_blocking_section();
-    ov_clear(df->ovf);
-    caml_leave_blocking_section();
-    free(df->ovf);
-    df->ovf=NULL;
-  }
-  if(df->read_func)
-  {
-    caml_remove_global_root(&df->read_func);
-    caml_remove_global_root(&df->seek_func);
-    caml_remove_global_root(&df->close_func);
-    caml_remove_global_root(&df->tell_func);
-    df->read_func=0;
-  }
+  ov_clear(df->ovf);
+  free(df->ovf);
+  df->ovf=NULL;
+  caml_remove_global_root(&df->read_func);
+  caml_remove_global_root(&df->seek_func);
+  caml_remove_global_root(&df->tell_func);
 
   free(df);
 }
@@ -555,17 +568,6 @@ static int seek_func_cb(void *datasource, ogg_int64_t offset, int whence)
   return ret;
 }
 
-static int close_func_cb(void *datasource)
-{
-  myvorbis_dec_file_t *df = datasource;
-
-  caml_leave_blocking_section();
-  caml_callback(df->close_func, Val_unit);
-  caml_enter_blocking_section();
-
-  return 0;
-}
-
 static long tell_func_cb(void *datasource)
 {
   myvorbis_dec_file_t *df = datasource;
@@ -582,13 +584,13 @@ static ov_callbacks callbacks =
 {
   .read_func = read_func_cb,
   .seek_func = seek_func_cb,
-  .close_func = close_func_cb,
+  .close_func = NULL,
   .tell_func = tell_func_cb
 };
 
-CAMLprim value ocaml_vorbis_open_dec_stream(value read_func, value seek_func, value close_func, value tell_func, value params)
+CAMLprim value ocaml_vorbis_open_dec_stream(value read_func, value seek_func, value tell_func, value params)
 {
-  CAMLparam5(read_func, seek_func, close_func, tell_func, params);
+  CAMLparam4(read_func, seek_func, tell_func, params);
   CAMLlocal1(block);
   int ret = 0;
   myvorbis_dec_file_t *df;
@@ -597,14 +599,12 @@ CAMLprim value ocaml_vorbis_open_dec_stream(value read_func, value seek_func, va
 
   df->ovf = (OggVorbis_File*)malloc(sizeof(OggVorbis_File));
   df->bitstream = 0;
-  df->read_func = read_func;
   caml_register_global_root(&df->read_func);
-  df->seek_func = seek_func;
+  df->read_func = read_func;
   caml_register_global_root(&df->seek_func);
-  df->close_func = close_func;
-  caml_register_global_root(&df->close_func);
-  df->tell_func = tell_func;
+  df->seek_func = seek_func;
   caml_register_global_root(&df->tell_func);
+  df->tell_func = tell_func;
 
   caml_enter_blocking_section();
   ret = ov_open_callbacks(df, df->ovf, NULL, 0, callbacks);
@@ -613,7 +613,6 @@ CAMLprim value ocaml_vorbis_open_dec_stream(value read_func, value seek_func, va
   if(ret < 0)
   {
     caml_remove_global_root(&df->tell_func);
-    caml_remove_global_root(&df->close_func);
     caml_remove_global_root(&df->seek_func);
     caml_remove_global_root(&df->read_func);
     free(df->ovf);
@@ -621,9 +620,7 @@ CAMLprim value ocaml_vorbis_open_dec_stream(value read_func, value seek_func, va
     raise_err(ret);
   }
 
-  block = caml_alloc_custom(&decfile_ops, sizeof(myvorbis_dec_file_t*),
-                            sizeof(myvorbis_dec_file_t)+sizeof(OggVorbis_File),
-                            1000000);
+  block = caml_alloc_custom(&decfile_ops, sizeof(myvorbis_dec_file_t*),0,1);
   Decfile_val(block) = df;
 
   CAMLreturn(block);
@@ -752,31 +749,6 @@ CAMLprim value ocaml_vorbis_decode_float_alloc(value d_f, value len_)
   CAMLreturn(ans);
 }
 
-CAMLprim value ocaml_vorbis_close_dec_file(value d_f)
-{
-  CAMLparam1(d_f);
-  myvorbis_dec_file_t* df = Decfile_val(d_f);
-
-  if (df->ovf)
-  {
-    caml_enter_blocking_section();
-    ov_clear(df->ovf);
-    caml_leave_blocking_section();
-    free(df->ovf);
-    df->ovf=NULL;
-  }
-  if(df->read_func)
-  {
-    caml_remove_global_root(&df->read_func);
-    caml_remove_global_root(&df->seek_func);
-    caml_remove_global_root(&df->close_func);
-    caml_remove_global_root(&df->tell_func);
-    df->read_func=0;
-  }
-
-  CAMLreturn(Val_unit);
-}
-
 CAMLprim value ocaml_vorbis_get_dec_file_bitstream(value d_f)
 {
   myvorbis_dec_file_t *df = Decfile_val(d_f);
@@ -948,7 +920,7 @@ CAMLprim value ocaml_vorbis_skeleton_fisbone(value serial, value samplerate, val
   memset (&op, 0, sizeof (op));
   op.packet = malloc(len);
   if (op.packet == NULL)
-    caml_failwith("malloc");
+    caml_raise_out_of_memory();
 
   memset (op.packet, 0, len);
   /* it will be the fisbone packet for the vorbis audio */
